/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.kernel.axiom.atomic;

import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.transport.Communication;

import java.util.HashMap;

/**
 * Flow are the high-level component to provide flow-related functionality. Every component is mandatorly assigned to a flow componen.
 * Flows shows tree structure, refining the overall processes of the given application.
 * According to the specification of the Brillien, there are several services provided by a flow:
 * - sharing
 * - acknowledgement about the commmunication of their components
 * - possible fault handling
 * - etc.
 */
public interface Flow<P extends Presence, C extends Communication, R> extends Context<P,C,R>{

    /**
     * @return the identifier of the flow instance
     */
    String                              getFlowID();

    /**
     * Setter method for the ID
     * @param flowID identifier of the given flow
     * @return the flow instance itself
     */
    Flow<P,C,R>                         setFlowID(String flowID);

    /**
     * Tells whether this Flow represents an action which cannot fail, so cluster replication is needed.
     */
    boolean                             isIndispensable();

    /**
     * Setter method for an indispensable flow
     */
    void                                setIndispensable( boolean indispensable );

    /**
     * Report method to share any communication related information of the components with the container flow instance.
     * @param reporter reporter Presence reference
     * @param message communication' message name
     * @param params message parameters
     */
    void                                report( Presence reporter, String message, Object... params );

    /**
     * Returns the result of the flow if it exists
     */
    <T> T                               getFlowResult();

    /**
     * Sets the result of the flow
     */
    void                                setFlowResult( Object result );

    /**
     * Activates the flow instance, making it started.
     * @throws BrillienException in case of operation failure
     */
    void                                activate( C c, Long timeout ) throws BrillienException;

    /**
     * Gets the parameters gotten while the given flow has been activated with.
     * @return
     */
    C                                   getActivationData();

    /**
     * Sets all state of a flow to "Error" state marking it to be terminated.
     * @throws BrillienException in case of operation failure
     */
    void                                oppress( String message ) throws BrillienException;

    /**
     * Sets all state of a flow to "Exit" state marking it to be terminated.
     * @throws BrillienException in case of operation failure
     */
    void                                terminate() throws BrillienException;

    /**
     * Tells whether the flow has been suspended.
     */
    boolean                             isSuspended();

    /**
     * All Indispensable flows can be suspended and resumed. This method makes all the necessary infromation (id, current states) persisted about a given flow.
     * Fot all non indispensable flow, an exception will be thrown. 
     * @throws BrillienException in case of suspend failure or the non-indispensable nature of the given flow instance
     */
    void                                suspend() throws BrillienException;

    /**
     * Notify the given flow instance about the occured revivfication process. 
     */
    void                                revivified() throws BrillienException;

    /**
     * This method retrieves the state space of the flow called when the flow will be suspended
     */
    HashMap<String, Object>             getStateSpace();

    /**
     * This method sets the state space of the flow called when the flow will be reactivated
     */
    void                                setStateSpace( HashMap<String, Object> stateSpace ) throws BrillienException;
}
